################################################################################
# FitUnifiedModel.R
# Code Author: Mizue Hisano, with edits by Sean Connolly
# Most recent changes: 24 January 2016 (tidy up for DRYAD)
# ----------------------------------------------------------------------
# Example code to fit the NB-LN-PL (unified model)and NB-G-PL models to 
# abundance data.
#
# If you use this code, please cite the following:
# Connolly et al. (2017) A unified model explains commonness and
# rarity on coral reefs. Ecology Letters.
#===============================================================================

# PRELIMINARIES
# -------------

library(poilog) # Not essential: used here to get initial parameter
                # parameter estimates for the metacommunity SAD for the 
                # NB-LN-PL model

# read in the abundance data
# species-by-site abundance matrix for a metacommunity
# e.g. IP Corals, Indonesia sites, slope-habitat
# We assume that the CSV file is in the folder "Abunds", and that R's
# working directory is the directory where "Abunds" sits.
abundmat <- as.matrix(read.csv("Abunds/Abunds_Corals_Slope_Indonesia.csv",
            header=T))

# read in the functions used to fit the NB-LN-PL model
source("NBLNPL_LLfxn.R")

# Find a set of reasonable initial par values to be passed onto nlminb
# ---------------------------------------------------------------------------

# For the power-law parameters, regress log(mean) against log(variance)
mus <- rowMeans(abundmat)
vars <- apply(abundmat,1,var)
mvfit <- lm(log(vars)~log(mus))
plini <- c(coef(mvfit)[1],log(coef(mvfit)[2]))
names(plini) = c("log(a)","log(b)")
# intercept (coefficient 1) is log(a) and coefficient 2 is b 
# (var = a*mean^b)

# For the metacommunity SAD, fit the poisson lognormal to the summed 
# metacommunity abundance.
require(poilog)
metafit = poilogMLE(rowSums(abundmat)) 
sitem = exp(metafit$par[1]-log(ncol(abundmat))) 
# rescale mu to site scale, and backtransform to arithmetic scale

lnini = c(sqrt(sitem),1/metafit$par[2])
names(lnini) = c("sqrt(m)","1/sig")
# These transformations have been chosen because they produce relatively 
# symmetric profile likelihoods across the different metacommunities analysed
# in the paper.

init <- c(plini,lnini)
# In practice, it is important to re-fit the model using different initial
# parameter estimates, to ensure that true MLEs are found

# Fit the NB-LN-PL model
# ---------------------------
fit.nblnpl <- nlminb(init,Fit.nbv.ln,lower=c(-Inf,-Inf,0,0),obs=abundmat)
if(fit.nblnpl$conv!=0) {
  # in case of convergence issue, re-start from similar ini pars
  fit.nblnpl <- nlminb(rnorm(length(init),mean=init,sd=abs(init/4)),Fit.nbv.ln,obs=abundmat)
}
print(c("convergence ",fit.nblnpl$conv))
 # Be sure to check here if the fit converged. If it didn't, rerun the previous
 # step.

# Back-transform parameter estimates. Note that the third parameter is the 
# mean of log-abundance for the SAD, BACK-TRANSFORMED to an arithmetic scale
mle.nblnpl <- c(exp(fit.nblnpl$par[1:2]),fit.nblnpl$par[3]^2,1/abs(fit.nblnpl$par[4]))
names(mle.nblnpl) = c("a","b","m","sig")

mll.nblnpl <- -fit.nblnpl$obj
rcom <- colSums(abundmat)/mean(colSums(abundmat))
  # a vector of relative community sizes
  
p0.nblnpl <- getP0.nbv.ln(mle.nblnpl[1],mle.nblnpl[2],mle.nblnpl[3],mle.nblnpl[4],rcom)
  # probability of observing 0 abundance at every site 
  
Spool.nblnpl <- nrow(abundmat)/(1-p0.nblnpl)
  # estimated community species-pool size

# read in the functions used to fit the NB-LN-PL model
source("NBGPL_LLfxn.R")

# fit the NB-G-PL model

init <- c(fit.nblnpl$par[1:2],0,0)

fit.nbgpl <- nlminb(init,Fit.nbv.gam,obs=abundmat)
if(fit.nbgpl$conv!=0)
  # in case of convergence issue, re-start from similar ini pars
  fit.nbgpl <- nlminb(rnorm(length(init),init),Fit.nbv.gam,obs=abundmat)
mle.nbgpl <- exp(fit.nbgpl$par)
mll.nbgpl <- -fit.nbgpl$obj
p0.nbgpl <- getP0.nbv.gam(mle.nbgpl[1],mle.nbgpl[2],mle.nbgpl[3],mle.nbgpl[4],rcom)
  # probability of observing 0 abundance at every site
Spool.nbgpl <- nrow(abundmat)/(1-p0.nbgpl)
  # estimated community species-pool size
