################################################################################
# GOFtest_UnifiedModel.R
# Code Author: Mizue Hisano, with edits by Sean Connolly
# Most recent changes: 24 January 2016 (tidy up for DRYAD)
# ----------------------------------------------------------------------
# Example code to conduct a parametric bootstrap GOF test for the unified model,
# as follows:
#
# 1. Using MLEs (a, b, m, sigma) and Sobs, calculate Spool = Sobs/(1-P0)
# 2. Randomly draw a species-specific mean abundance mu_i from the MLEs of
#    (mu, sigma), for each species
# 3. Find a vector of species-and-site-specific mean abundances mus_ij using
#     a vector of the observed relative sample sizes across sites
#     (mus_ij = rcom * mu_i)
# 4.a If var = a*mu_i^b > mu_i, randomly draw an abundance value for species i
#     at each site j from a NB distribution with mean mus_ij and MLE of a, b
#     (k = mu_i^2/(a*mu_i^b-mu_i))
# 4.b If var = a*mu_i^b <= mu_i, randomly draw an abundance value for species i 
#     at each site j from a Poisson distribution with mean mus_ij
# 5. Eliminate all species that have abundance 0 everywhere
# 6. Fit the NB-LN-PL model to the simulated data
# 7. Repeat steps 1000 times
# 8. Calculate goodness-of-fit (C-hat) statistics
# 9. Calculate community similarity (Horn index) statistics

# If you use this code, please cite the following:
# Connolly et al. (2017) A unified model explains commonness and
# rarity on coral reefs. Ecology Letters.
#===============================================================================



################################################################################

# set the number of simulations
# IT IS STRONGLY RECOMMENDED to initially set this to a very small value
# (say, numsim=1, then numsim=10) to make sure it works reasonably well
# and produces parameter estimates that converge, before running
# a large number of simulations!!!
numsim <- 1000

# Read in the data and get fit the unified model to it
# (Be sure to edit FitUnifiedMOdel.R so that it reads the correct CSV file)
source("FitUnifiedModel.R")
a.mle <- mle.nblnpl[1]
b.mle <- mle.nblnpl[2]
m.mle <- mle.nblnpl[3]
sig.mle <- mle.nblnpl[4]

# number of observed species
Sobs <- nrow(abundmat)

# number of sites in the sample
nsites <- ncol(abundmat)

# simulate data
set.seed(1)
simdata <- array(dim=c(round(Spool.nblnpl),nsites,numsim))
for(isim in 1:numsim) {
  # randomly draw a mean abundance using the fitted m and s
  mu.i <- rlnorm(round(Spool.nblnpl),meanlog=log(m.mle),sdlog=sig.mle)
  # using the observed relative sample sizes across sites of this group,
  # calculate species-and-site-specific mean abundance values
  mus.ij <- sapply(mu.i,'*',rcom)
  for(isp in 1:length(mu.i)) {
    # for each species i,
    # if var > mean (i.e. a*mu_i^b > mu_i --> mu_i > exp(log(a)/(1-b))
    # randomly draw an abundance value at each site j from a NB with
    # the sampled, adjusted mean mus.ij and the fitted aggregation parameter
    # k = mu_i^2/(a*mu_i^b-mu_i)
    if(mu.i[isp] > exp(log(a.mle)/(1-b.mle))) {
      k.i <- mu.i[isp]^2/(a.mle*mu.i[isp]^b.mle-mu.i[isp])
      simdata[isp,,isim] <- rnbinom(nsites,size=k.i,mu=mus.ij[,isp])
    } else
    # if var <= mean (i.e. a*mu_i^b <= mu_i --> mu_i <= exp(log(a)/(1-b))
    # randomly draw an abundance value at each site j from a Poisson with
    # the sampled adjusted mean mus.ij
      simdata[isp,,isim] <- rpois(nsites,lambda=mus.ij[,isp])
  }
}


# fit the NB-LN-PL model to simulated data

NBLNPL.sim <- matrix(nrow=9,ncol=numsim)
rownames(NBLNPL.sim) <- c("a","b","m","sig","p0","negLL","conv","numS","totN")
for(isim in 1:numsim) {
  print(paste("fitting the model to sim",isim)); flush.console()
  # initialize based on the original MLEs
  init = c(log(a.mle),log(b.mle),sqrt(m.mle),1/sig.mle)

  # extract the relevant simulated count data
  abundmat.sim <- simdata[,,isim]
  # get rid of species that haven't been observed
  abundmat.sim <- abundmat.sim[rowSums(abundmat.sim)!=0,]
  # fit the model to obs
  fit.sim <- try(nlminb(init,Fit.nbv.ln,lower=c(-Inf,-Inf,0,0),obs=abundmat.sim),TRUE)
  while(class(fit.sim)=="try-error") {
    # Depending on the data set, this may need some tweaking
    print("re-fitting with a slightly different inipars"); flush.console()
    fit.sim <- try(nlminb(rnorm(4,mean=init,sd=0.5),Fit.nbv.ln,lower=c(-Inf,-Inf,0,0),obs=abundmat.sim),TRUE)
  }
  MLEs <- c(exp(fit.sim$par[1:2]),fit.sim$par[3]^2,1/fit.sim$par[4])
  NBLNPL.sim[1:4,isim] <- MLEs
  rcomvec = colSums(abundmat.sim)/mean(colSums(abundmat.sim))
  NBLNPL.sim[5,isim] <- getP0.nbv.ln(MLEs[1],MLEs[2],MLEs[3],MLEs[4],rcomvec)
  NBLNPL.sim[6,isim] <- fit.sim$obj
  NBLNPL.sim[7,isim] <- fit.sim$conv
  NBLNPL.sim[8,isim] <- nrow(abundmat.sim)
  NBLNPL.sim[9,isim] <- sum(abundmat.sim)
}


# goodness-of-fit (c-hat) calculation

# find all the unique vector of counts in abundmat
rvecs <- unique(abundmat)
# find how many species have abundance vector equal to each row of rvecs
Srvecs <- apply(rvecs,1,function(r)sum(apply(abundmat,1,function(x)identical(x,r))))
# calculate saturated loglikelihood
SLL <- sum(Srvecs*log(Srvecs/Sobs))
# calculate deviances: dev = 2*(SLL - MLL_NBLNPL)
dev.obs <- 2*(SLL - mll.nblnpl)
dev.sim <- rep(NA,numsim)
for(isim in 1:numsim) {
  # extract the relevant simulated count data
  abundmat.sim <- simdata[,,isim]
  # get rid of species that haven't been observed
  abundmat.sim <- abundmat.sim[rowSums(abundmat.sim)!=0,]
  # total number of observed species
  Sobs.sim <- nrow(abundmat.sim)
  # find all the unique vector of counts in abundmat.sim
  rvecs.sim <- unique(abundmat.sim)
  # find how many species have abundance vector equal to each row of rvecs
  Srvecs.sim <- apply(rvecs.sim,1,function(r)sum(apply(abundmat.sim,1,
                function(x)identical(x,r))))
  # saturated loglikelihood
  SLL <- sum(Srvecs.sim*log(Srvecs.sim/Sobs.sim))
  # calculate deviances: dev = 2*(SLL - MLL_NBLNpl)
  dev.sim[isim] <- 2*(SLL - (-NBLNPL.sim["negLL",isim]))
}

# calculate C-hat = dev.obs / dev.sim
chat <- dev.obs/dev.sim


# Horn index calculation

# this function takes two vectors of abundances and calculates Horn index
CalculateHornIndx <- function(abund1,abund2) {
  # obs1: a vector of abundances for the 1st assemblage
  # obs2: a vector of abundances for the 2nd assemblage
  
  # only count species that appear in at least one assemblage
  keep <- abund1>0|abund2>0
  abund1 <- abund1[keep]
  abund2 <- abund2[keep]
  
  # calculate p(i;1) = relative abundance of ith species in 1st assemblage.
  #
  # for those rare cases where all abund =0 for a site,
  # assume relative abundances of all species are equal
  # (This may happen for simulated communities where there are a large
  # number of estimated unobserved species -- such as fits of the NBGPL
  # model. An alternative might be to simply assess such a community's
  # similarity with any other communities to be zero.)
  if(all(abund1==0))
    pi1 <- rep(1/length(abund1),length(abund1)) else
    pi1 <- abund1/sum(abund1)
  if(all(abund2==0))
    pi2 <- rep(1/length(abund2),length(abund2)) else
    pi2 <- abund2/sum(abund2)
  part1 <- rep(0,length(pi1))
  part2 <- rep(0,length(pi2))
  part1[pi1>0] <- pi1[pi1>0]/2 * log(1+pi2[pi1>0]/pi1[pi1>0])
  part2[pi2>0] <- pi2[pi2>0]/2 * log(1+pi1[pi2>0]/pi2[pi2>0])
  H = 1/log(2) * sum(part1+part2)
  return(H)
}

# calculate Horn index for each pair of sites
HornIndx <- combn(1:nsites,2,FUN=function(y)
            CalculateHornIndx(abundmat[,y[1]],abundmat[,y[2]]))

# repeat for the simulated data
HornIndx.sim <- matrix(nrow=choose(nsites,2),ncol=numsim)
for(isim in 1:num sim) {
  # extract the relevant simulated abundance data
  abundmat.sim <- simdata[,,isim]
  # get rid of species that haven't been observed
  abundmat.sim <- abundmat.sim[rowSums(abundmat.sim)!=0,]
  # calculate Horn's index between pairs of sites
  HornIndx.sim[,isim] <- combn(1:nsites,2,FUN=function(y)
                    CalculateHornIndx(abundmat.sim[,y[1]],abundmat.sim[,y[2]]))
}


# use these Horn index values to test the null hypothesis that
# Horn indices of the observed and simulated counts are drawn from
# the same distribution

# do this by testing if the cumulative fractions of Horn index across
# specific pairs of sites are different between observed and simulated datasets

# find the average cumulative fractions of simulated Horn index values
GetMeanSim <- function(ys) {
  # find the average cummulative fractions from
  # simulated Horn index values (ys) at each unique value in ys
  # ys: a matrix of simulated Horn index values (ith pair of sites x numsim)
  ys <- apply(ys,2,sort)
  # find unique values in ys
  w <- sort(unique(c(ys)))
  # at each w, calculate the average cummulative fraction.
  # first, find the number of steps taken at each value of w, for each y vector
  # calculate cumulative fractions for each vector of ys at each w
  cdf.w <- sapply(1:ncol(ys),function(i)ecdf(ys[,i])(w))
  av.cdf.w <- rowMeans(cdf.w)
  return(list(y=w,cdf=av.cdf.w))
}
Ey <- GetMeanSim(HornIndx.sim)

# calculate K-S statistic for
# observed vs Ey, and simulated vs Ey
KSstat <- function(x,y,cdf.y=NULL) {
  # x, y : two vectors of values to calculate K-S statistic for
  x <- sort(x)
  w <- sort(unique(c(x,y)))
  # calculate cumulative fractions for x at each w
  xval <- ecdf(x)(w)
  ks.test(xval,cdf.y,alternative="two.sided")
}

obs <- KSstat(HornIndx,Ey$y,Ey$cdf)
D.obs <- obs$stat
sim <- apply(HornIndx.sim,2,function(y)KSstat(y,Ey$y,Ey$cdf))
D.sim <- unlist(lapply(sim,function(si)si$stat))

# calculate the percentile of D.obs in D.sim to rank where D.obs sits
# (one-tail: testing if D.obs is greater than D.sim)
Perc.obs <- sum(D.sim<D.obs)/numsim
# converting this into p-value (if Perc.obs>0.95, p-value is significant)
P.obs <- 1-Perc.obs

