################################################################################
# NBGPL_LLfxn.R
# Code Author: Mizue Hisano, with edits by Sean Connolly
# Most recent changes: 24 January 2016 (tidy up for DRYAD)
# ----------------------------------------------------------------------
# Functions called by FitUnifiedModel.R to fit the NB-Gamma alternative to the 
# unified model (gamma metacommunity SAD, negative binomial distribuiton of
# abundances across sites within species, and power-law mean-variance scaling
# within species across sites). This is fitted to a sites columns) x species 
# (rows) matrix of abundances (counts of individuals).
#
# If you use this code, please cite the following:
# Connolly et al. (2017) A unified model explains commonness and
# rarity on coral reefs. Ecology Letters.
#===============================================================================

Fit.nbv.gam = function(pars, obs) {
  # function passed to an optimiser for fitting of the NB-G-PL model
  # parameters are:
  # a,b = the parameters that shape the variance for the negative binomial
  # var = a*mu^b
  # shape,scal = shape and scale of the gamma regional SAD
  # obs = a matrix of abundances with columns for sites and rows for species
  a = exp(pars[1])
  b = exp(pars[2])
  shape = exp(pars[3])
  scal = exp(pars[4])
  # number of sites
  S = ncol(obs)

  # relative community sizes
  rcom = colSums(obs)/mean(colSums(obs))
    # Note rcom is scaled so that it has a mean value of 1 (this is necessary
    # so that the calculated variance is based on site-level mean abundance --
    # specifically the species mean abundance at average community size.

  # calculate p0, probability of not observing any species at any sites
  p0 = getP0.nbv.gam(a,b,shape,scal,rcom)
  LL = rep(NA,nrow(obs))
  for (iSpecies in 1:nrow(obs)) {
    # calculate the likelihood for one species
    # vector of abundances observed for this species
    ab = obs[iSpecies,]
    loglikeli = LL.nbv.gam(a,b,shape,scal,ab,rcom)
    # add normalization factor since we condition on the species being
    # observed in at least one site:
    LL[iSpecies] = loglikeli - log(1-p0)
  }
  # return the negative log-likelihood
  return(-sum(LL))
}

getP0.nbv.gam = function(a, b, shape, scal, rcom) {
  # the lower part of the integral (poisson-gamma part)
  # is calculated by difference instead of integrating from 0 to the threshold

  # mu = mean for species i on the arithmetic scale
  # a,b = power-law parameters
  # shape, scal = shape and scale of the gamma regional SAD
  # rcom = vector of relative community sizes (average is 1)

  # number of sites
  S = length(rcom)

  thresh = exp(log(a)/(1-b)) # obtained by solving a*mu^b = mu for mu  -- cutoff
                             # for transition between poisson vs power-law NB
                             # spatial distribution of abundances
  # if b>1 use pois below threshold & nb above threshold
  if(b>1) {
    int.nb = integrate(fnbvgam,lower=thresh,upper=Inf,a=a,b=b,
              shape=shape,scal=scal,ab=rep(0,S),rcom=rcom)$value
    int.poi = try(integrate(fpoivgam,lower=0,upper=thresh,shape=shape,scal=scal,
              ab=rep(0,S),rcom=rcom)$value,T)
      # this can give an error with a message that the integrant may be divergent
    if(class(int.poi)=="try-error") {
      # try taking a part of dgamma out of the integral
      # and calculating the difference between
      # all: integration from 0 -> Inf, and
      # up: integration from thresh -> Inf
      int.poi.part1 = 1/scal^(shape)
      int.poi.all = try(integrate(fpoivgam.try,lower=0,upper=Inf,
              shape=shape,scal=scal,ab=rep(0,S),rcom=rcom)$value,T)
      int.poi.up = try(integrate(fpoivgam.try,lower=thresh,upper=Inf,
              shape=shape,scal=scal,ab=rep(0,S),rcom=rcom)$value,T)
      int.poi.part2 = int.poi.all-int.poi.up
      int.poi = int.poi.part1 * int.poi.part2
    }
  } else {
  # if b<1 use nb below threshold & pois above threshold
    int.nb = integrate(fnbvgam,lower=0,upper=thresh,a=a,b=b,
              shape=shape,scal=scal,ab=rep(0,S),rcom=rcom)$value
    int.poi = try(integrate(fpoivgam,lower=thresh,upper=Inf,shape=shape,scal=scal,
              ab=rep(0,S),rcom=rcom)$value,T)
      # this can give an error with a message that the integrant may be divergent
    if(class(int.poi)=="try-error") {
      # try taking a part of dgamma out of the integral
      # and calculating the difference between
      # all: integration from 0 -> Inf, and
      # down: integration from 0 -> thresh
      int.poi.part1 = 1/scal^(shape)
      int.poi.all = try(integrate(fpoivgam.try,lower=0,upper=Inf,
              shape=shape,scal=scal,ab=rep(0,S),rcom=rcom)$value,T)
      int.poi.down = try(integrate(fpoivgam.try,lower=0,upper=thresh,
              shape=shape,scal=scal,ab=rep(0,S),rcom=rcom)$value,T)
      int.poi.part2 = int.poi.all-int.poi.down
      int.poi.part2 = max(0,int.poi.part2)
      int.poi = int.poi.part1 * int.poi.part2
    }
  }
  # return the probability
  return(int.nb + int.poi)
}

LL.nbv.gam = function(a,b,shape,scal,ab,rcom) {
  # function to calculate the log-likelihood of observing abundance vector "ab"
  # for a species, given parameters a,b,shape,scal
  # NOTE that this function does not include the zero-vector truncation, so
  # is not called directly from an optimizer

  # a,b = power-law scaling parameters
  # shape,scal = shape and scale of the gamma regional SAD
  # ab = vector of abundances observed for this species
  # rcom = vector of relative community sizes

  thresh = exp(log(a)/(1-b)) # obtained by solving a*mu^b = mu for mu

  if(b>1) {
    # use NB above threshold of var = mean
    # use Pois below threshold
    int.nb = integrate(fnbvgam,lower=thresh,upper=Inf,
             a=a,b=b,shape=shape,scal=scal,ab=ab,rcom=rcom)$value
    int.poi = try(integrate(fpoivgam,lower=0,upper=thresh,
              shape=shape,scal=scal,ab=ab,rcom=rcom)$value,T)
    # this can give an error with a message that the integrant may be divergent
    if(class(int.poi)=="try-error") {
      # try taking a part of dgamma out of the integral
      # and calculating the integral part from the difference between
      # all: integration from 0 -> Inf, and
      # up: integration from thresh -> Inf
      int.poi.part1 = 1/scal^(shape)
      int.poi.all = try(integrate(fpoivgam.try,lower=0,upper=Inf,
                    shape=shape,scal=scal,ab=ab,rcom=rcom)$value,T)
      int.poi.up = try(integrate(fpoivgam.try,lower=thresh,upper=Inf,
                   shape=shape,scal=scal,ab=ab,rcom=rcom)$value,T)
      int.poi.part2 = int.poi.all-int.poi.up
      int.poi.part2 = max(0,int.poi.part2)
      int.poi = int.poi.part1 * int.poi.part2
    }
  } else {
    # use NB below threshold of var = mean
    # use Pois above threshold
    int.nb = integrate(fnbvgam,lower=0,upper=thresh,
             a=a,b=b,shape=shape,scal=scal,ab=ab,rcom=rcom)$value
    int.poi = try(integrate(fpoivgam,lower=thresh,upper=Inf,
              shape=shape,scal=scal,ab=ab,rcom=rcom)$value,T)
    # this can give an error with a message that the integrant may be divergent
    if(class(int.poi)=="try-error") {
      # try taking a part of dgamma out of the integral
      # and calculating the integral part from the difference between
      # all: integration from 0 -> Inf, and
      # down: integration from 0 -> thresh
      int.poi.part1 = 1/scal^(shape)
      int.poi.all = try(integrate(fpoivgam.try,lower=0,upper=Inf,
                    shape=shape,scal=scal,ab=ab,rcom=rcom)$value,T)
      int.poi.down = try(integrate(fpoivgam.try,lower=0,upper=thresh,
                     shape=shape,scal=scal,ab=ab,rcom=rcom)$value,T)
      int.poi.part2 = int.poi.all-int.poi.down
      int.poi.part2 = max(0,int.poi.part2)
      int.poi = int.poi.part1 * int.poi.part2
    }
  }
  # calculate the total loglikelihood
  loglikeli = log(int.nb + int.poi)

  # return the likelihood
  return(loglikeli)
}

# for the negative binomial part of the integral
fnbvgam = function(mu, a, b, shape, scal, ab, rcom) {
  # Note this function is integrated only for mean abundances mu for which
  # the variance (a*mu^b) exceeds the mean.

  # mu = mean for species i on the arithmetic scale
  # shape, scal = scale and shape of the gamma distribution
  # ab = vector of abundances
  # rcom = vector of relative community sizes (average is 1)

  v = a*mu^b  # note variance is assumed constant across sites,
               # based on estimated species' mean abundance at
               # the average local community size
  k = mu^2/(v-mu)   # likewise k constant across sites
  mus = sapply(rcom,'*',mu)   # Different means for each site:
    # number of rows equals length of mu vector passed by integrate
    # number of columns equals to the number of sites
  if(length(rcom)>1)
    nbpart = colSums(sapply(1:length(mu),function(i)
              dnbinom(ab,size=k[i],mu=mus[i,],log=T))) else
    nbpart = sapply(1:length(mu),function(i)
              dnbinom(ab,size=k[i],mu=mus[i,],log=T))

  # return the probability
  exp(nbpart + dgamma(mu,shape=shape,scale=scal,log=T))
}

# for the poisson part of the integral
fpoivgam = function(mu, shape, scal, ab, rcom) {
  # This is used whenever the power law would predict
  # variance less than the mean -- thus we implicitly
  # assume that variance=mean for any mean abundance
  # below the threshold where mu < exp( log(a)/(1-b) )

  # mu = mean for species i on the arithmetic scale
  # shape, scal = shape and scale parameters of the gamma regional SAD
  # ab = vector of abundances observed for this species
  # rcom = vector of relative community sizes

  # let the value of mu vary according to the relative sample size of each site
  # but keep the average at mu
  mus = sapply(rcom,'*',mu)
    # the average of rcom is 1, so the average of mus is mu
  if(length(rcom)>1)
    poispart = colSums(apply(mus,1,function(i)dpois(ab,i,log=T))) else
    poispart = apply(mus,1,function(i)dpois(ab,i,log=T))

  # return probability
  exp(poispart + dgamma(mu,shape=shape,scale=scal,log=T))
}

fpoivgam.try = function(mu, shape, scal, ab, rcom) {
  # This is used as a second trial at calculating the
  # integral when the power law would predict variance less than the mean.
  # One term (scale^shape) from dgamma() has been taken out of this integral

  # mu = mean for species i on the arithmetic scale
  # shape, scal = shape and scale parameters of the gamma regional SAD
  # ab = vector of abundances observed for this species
  # rcom = vector of relative community sizes

  # let the value of mu vary according to the relative sample size of each site
  # but keep the average at mu
  mus = sapply(rcom,'*',mu)
    # the average of rcom is 1, so the average of mus is mu
  if(length(rcom)>1)
    poispart = colSums(apply(mus,1,function(i)dpois(ab,i,log=T))) else
    poispart = apply(mus,1,function(i)dpois(ab,i,log=T))
  if(length(rcom)>1)
    gampart = colSums(apply(mus,1,function(i)
                -lgamma(shape)+(shape-1)*log(i)-i/scal)) else
    gampart = apply(mus,1,function(i)-lgamma(shape)+(shape-1)*log(i)-i/scal)

  # return probability
  exp(poispart + gampart)
}


