################################################################################
# OutOfSamplePrediction_UnifiedModel.R
# Code Author: Mizue Hisano, with edits by Sean Connolly
# Most recent changes: 24 January 2016 (tidy up for DRYAD)
# ----------------------------------------------------------------------
# Example code to generate out-of-sample predictions for the unified model
# Basic procedure:
# 1. put one site aside
# 2. fit model to the remaining sites
# 3. repeat for each site in turn
# 4. For each fit, predict the number of previously observed species that
#    will be missing at the excluded site.
#
# If you use this code, please cite the following:
# Connolly et al. (2017) A unified model explains commonness and
# rarity on coral reefs. Ecology Letters.
#===============================================================================

# read in the abundance data
# species-by-site abundance matrix for a metacommunity
# e.g. IP Corals, Indonesia sites, slope-habitat
# We assume that the CSV file is in the folder "Abunds", and that R's
# working directory is the directory where "Abunds" sits.
abundmat <- as.matrix(read.csv("Abunds/Abunds_Corals_Slope_Indonesia.csv",
            header=T))

# read in the functions used to fit the NB-LN-PL model
source("NBLNPL_LLfxn.R")

#===============================================================================

# setting the initial parameter values to pass onto nlminb
# (see FitUnifiedModel.R for details)
mus <- rowMeans(abundmat)
vars <- apply(abundmat,1,var)
mvfit <- lm(log(vars)~log(mus))
plini <- c(coef(mvfit)[1],log(coef(mvfit)[2]))
names(plini) = c("log(a)","log(b)")
require(poilog)
metafit = poilogMLE(rowSums(abundmat)) 
sitem = exp(metafit$par[1]-log(ncol(abundmat))) 
lnini = c(sqrt(sitem),1/metafit$par[2])
names(lnini) = c("sqrt(m)","1/sig")
init.orig <- c(plini,lnini)
fit.nblnpl <- nlminb(init.orig,Fit.nbv.ln,lower=c(-Inf,-Inf,0,0),obs=abundmat)
init <- fit.nblnpl$par

# fit the NB-LN-PL model
# setting aside each site in turn
fits.nblnpl.sub <- matrix(nrow=ncol(abundmat),ncol=9)
colnames(fits.nblnpl.sub) <- c("a","b","m","sig","p0","negLL","conv","numS","totN")
for(isite in 1:ncol(abundmat)) {
  (print(paste("Excluding site",isite)))
  abund.sub <- abundmat[,-isite]
  # omit species that are all 0
  abund.sub <- abund.sub[rowSums(abund.sub>0)>0,]
  fit.nblnpl.sub <- nlminb(init,Fit.nbv.ln,lower=c(-Inf,-Inf,0,0),obs=abund.sub)
  a <- exp(fit.nblnpl.sub$par[1])
  b <- exp(fit.nblnpl.sub$par[2])
  m <- fit.nblnpl.sub$par[3]^2
  sig <- 1/fit.nblnpl.sub$par[4]
  rcom = colSums(abund.sub)/mean(colSums(abund.sub))
    # relative community abundances
  p0 <- getP0.nbv.ln(a, b, m, sig, rcom)
  negLL <- fit.nblnpl.sub$obj
  conv <- fit.nblnpl.sub$conv
  numS <- nrow(abund.sub)
  totN <- sum(abund.sub)
  fits.nblnpl.sub[isite,] <- c(a,b,m,sig,p0,negLL,conv,numS,totN)
}


#===============================================================================

# predict the number of previously observed species that is missing for
# the site that was excluded from the corresponding fit

fnbv.0.missingsp = function(mu, a, b, m, sig, indx12, rcom) {
  # Note this function is integrated only for mean abundances mu for which
  # the variance (a*mu^b) exceeds the mean.

  # calculate for a vector of mu,
  # probabilities of previously observed species missing from the site that was
  # left out in fitting of the model

  # mu = mean for species i on the arithmetic scale (average across sites)
  # log(m), sig = parameters of the lognormal distribution
  # indx12 = an index indicating which position the site that was left out of
  #          fitting belongs
  # rcom = vector of relative community sizes
  #        (average is 1 for the 11 sites used for model fitting)

  v = a*mu^b   # note variance is assumed constant across sites,
               # based on estimated species' mean abundance at
               # the average local community size
  k = mu^2/(v-mu)   # likewise k constant across sites
  mus = sapply(rcom,'*',mu)   # Different means for each site:
    # numrows equals length of mu vector passed by integrate
    # ncols is equal to the number of sites
      
  nbpart.0 <- sapply(1:length(mu),function(i)
            dnbinom(rep(0,length(rcom)),size=k[i],mu=mus[i,],log=T))
  # This quantity is the probability that a species has abundance zero at
  # each site (fitted and excluded), conditional on it's mean abundance
  
  nbpart.notall0 <- log(1-exp(colSums(nbpart.0[-indx12,])))
  # This calculates the log of the probability that a species is not from all 
  # of the sites used in the fitting (i.e., it does not have abundance zero
  # at all of those sites).
  
  nbpart.full <- nbpart.notall0+nbpart.0[indx12,]
  # This takes the above, and adds the log of the probability that a species
  # has zero abundance at the site that's been excluded from the fitting
  
  exp(nbpart.full + dlnorm(mu,log(m),sig,log=T))
  # Finally, add the probability density for mean abundance mu (we'll integrate
  # over this density)
  
}
fpoiv.0.missingsp = function(mu, m, sig, indx12, rcom) {
  # This is used whenever the power-law would predict
  # variance less than the mean -- thus we implicitly
  # assume that variance=mean for any mean abundance
  # below the threshold wherever mu < exp( log(a)/(1-b) )

  # calculate for a vector of mu,
  # probabilities of previously observed species missing from the site that was
  # left out in fitting of the model

  # mu = mean for species i on the arithmetic scale (average across sites)
  # log(m), sig = parameters of the lognormal distribution
  # indx12 = an index indicating which position the site that was left out of
  #          fitting belongs
  # rcom = vector of relative community sizes
  #        (average is 1 for the 11 sites used for model fitting)

  # let the value of mu vary according to the relative sample size of each site
  # but keep the average at mu
  mus = sapply(rcom,'*',mu)
    # the average of rcom is 1, so the average of mus is mu
  
  # These are the poisson analogues of the negative binomial terms calculated
  # (and commented in detail) in the preceding function.
  poispart.0 <- apply(mus,1,function(i)dpois(rep(0,length(rcom)),i,log=T))
  poispart.notall0 <- log(1-exp(colSums(poispart.0[-indx12,])))
  poispart.full <- poispart.notall0+poispart.0[indx12,]
  exp(poispart.full + dlnorm(mu,log(m),sig,log=T))
}
getP.missingsp <- function(a,b,m,sig,indx12,rcom) {
  # calculate and return probability of not observing
  # a previously observed species in the next site

  # indx12 = an index indicating which position the site that was left out of
  #          fitting belongs

  thresh = exp(log(a)/(1-b)) # obtained by solving a*mu^b = mu for mu  -- cutoff
                             # for transition between poisson vs power-law NB
                             # spatial distribution of abundances
  
  # Now we integrate over the distribution of metacommunity mean
  # abundances (i.e., the metacommunity SAD).
  
  # if b>1 use pois below threshold & nb above threshold
  if(b>1) {
    int.up = integrate(fnbv.0.missingsp,lower=thresh,upper=Inf,a=a,b=b,m=m,sig=sig,
              indx12=indx12,rcom=rcom)$value
    int.low = integrate(fpoiv.0.missingsp,lower=0,upper=thresh,m=m,sig=sig,
              indx12=indx12,rcom=rcom)$value
  } else {
  # if b<1 use nb below threshold & pois above threshold
    int.up = integrate(fnbv.0.missingsp,lower=0,upper=thresh,a=a,b=b,m=m,sig=sig,
              indx12=indx12,rcom=rcom)$value
    int.low = integrate(fpoiv.0.missingsp,lower=thresh,upper=Inf,m=m,sig=sig,
              indx12=indx12,rcom=rcom)$value
    if(int.up==0&int.low==0) {
      int.up = integrate(fnbv.0.missingsp,lower=0,upper=Inf,a=a,b=b,m=m,sig=sig,
              indx12=indx12,rcom=rcom)$value
    }
  }
  return(int.up + int.low)
}
#===============================================================================

# Using the fits of the model, calculate
# 1. probability of not observing a previously observed species in a new sample
# 2. estimated species pool size
Pmissingsp <- rep(NA,ncol(abundmat))
Spool <- rep(NA,ncol(abundmat))
for(isite in 1:ncol(abundmat)) {

  fit <- fits.nblnpl.sub[isite,]
  a <- fit["a"]
  b <- fit["b"]
  m <- fit["m"]
  sig <- fit["sig"]
  rcom <- colSums(abundmat)/mean(colSums(abundmat[,-isite]))
  Pmissingsp[isite] <- getP.missingsp(a,b,m,sig,isite,rcom)

  # estimated Spool from the fit to 11 sites
  # Spool = Sobs/(1-p0)
  Spool[isite] <- fit["numS"]/(1-fit["p0"])
}

# calculate the expected number of species that were previously observed but
# not in the next sample
ENmissingsp <- Pmissingsp*Spool

